@@ -42,10 +42,15 @@ def generate_MIMO_channel(raydata, params, tx_ant_params, rx_ant_params):
         
         array_response_TX = array_response(ant_tx_ind, 
                                            raydata[i][c.OUT_PATH_DOD_THETA], 
-                                           raydata[i][c.OUT_PATH_DOD_PHI], kd_tx)
+                                           raydata[i][c.OUT_PATH_DOD_PHI], 
+                                           kd_tx,
+                                           tx_ant_params[c.PARAMSET_ANT_ROTATION])
+        
         array_response_RX = array_response(ant_rx_ind, 
                                            raydata[i][c.OUT_PATH_DOA_THETA], 
-                                           raydata[i][c.OUT_PATH_DOA_PHI], kd_rx)
+                                           raydata[i][c.OUT_PATH_DOA_PHI], 
+                                           kd_rx,
+                                           rx_ant_params[c.PARAMSET_ANT_ROTATION][i])
         
         if  params[c.PARAMSET_FDTD]: # OFDM
             path_const = path_gen.generate(raydata[i][c.OUT_PATH_RX_POW].reshape(-1, 1), 
@@ -54,12 +59,12 @@ def generate_MIMO_channel(raydata, params, tx_ant_params, rx_ant_params):
             
             # The next step is to be defined
             if params[c.PARAMSET_OFDM][c.PARAMSET_OFDM_PULSE] == 0:
-                channel[i] = np.sum(array_response_RX[:, None, None, :] * np.conj(array_response_TX)[None, :, None, :] * path_const.T[None, None, :, :], axis=3)
+                channel[i] = np.sum(array_response_RX[:, None, None, :] * array_response_TX[None, :, None, :] * path_const.T[None, None, :, :], axis=3)
             else:
-                channel[i] = np.sum(array_response_RX[:, None, None, :] * np.conj(array_response_TX)[None, :, None, :] * path_const.T[None, None, :, :], axis=3)@path_gen.delay_to_OFDM
+                channel[i] = np.sum(array_response_RX[:, None, None, :] * array_response_TX[None, :, None, :] * path_const.T[None, None, :, :], axis=3)@path_gen.delay_to_OFDM
         
         else: # TD channel
-            channel[i, :, :, :raydata[i][c.OUT_PATH_NUM]] = array_response_RX[:, None, :] * np.conj(array_response_TX)[None, :, :] * (np.sqrt(raydata[i][c.OUT_PATH_RX_POW]) * np.exp(1j*np.deg2rad(raydata[i][c.OUT_PATH_PHASE])))[None, None, :]
+            channel[i, :, :, :raydata[i][c.OUT_PATH_NUM]] = array_response_RX[:, None, :] * array_response_TX[None, :, :] * (np.sqrt(raydata[i][c.OUT_PATH_RX_POW]) * np.exp(1j*np.deg2rad(raydata[i][c.OUT_PATH_PHASE])))[None, None, :]
 
     return channel
 
@@ -90,12 +95,18 @@ def generate_MIMO_channel_rx_ind(raydata, params, tx_ant_params, rx_ant_params):
         if raydata[i][c.OUT_PATH_NUM]==0:
             channel.append(None)
         
+        
         array_response_TX = array_response(ant_tx_ind, 
                                            raydata[i][c.OUT_PATH_DOD_THETA], 
-                                           raydata[i][c.OUT_PATH_DOD_PHI], kd_tx)
+                                           raydata[i][c.OUT_PATH_DOD_PHI], 
+                                           kd_tx,
+                                           tx_ant_params[c.PARAMSET_ANT_ROTATION])
+        
         array_response_RX = array_response(ant_rx_ind, 
                                            raydata[i][c.OUT_PATH_DOA_THETA], 
-                                           raydata[i][c.OUT_PATH_DOA_PHI], kd_rx)
+                                           raydata[i][c.OUT_PATH_DOA_PHI], 
+                                           kd_rx,
+                                           rx_ant_params[i][c.PARAMSET_ANT_ROTATION])
         
         if  params[c.PARAMSET_FDTD]: # OFDM
             path_const = path_gen.generate(raydata[i][c.OUT_PATH_RX_POW].reshape(-1, 1), 
@@ -104,24 +115,31 @@ def generate_MIMO_channel_rx_ind(raydata, params, tx_ant_params, rx_ant_params):
             
             # The next step is to be defined
             if params[c.PARAMSET_OFDM][c.PARAMSET_OFDM_PULSE] == 0:
-                channel.append(np.sum(array_response_RX[:, None, None, :] * np.conj(array_response_TX)[None, :, None, :] * path_const.T[None, None, :, :], axis=3))
+                channel.append(np.sum(array_response_RX[:, None, None, :] * array_response_TX[None, :, None, :] * path_const.T[None, None, :, :], axis=3))
             else:
-                channel.append(np.sum(array_response_RX[:, None, None, :] * np.conj(array_response_TX)[None, :, None, :] * path_const.T[None, None, :, :], axis=3)@path_gen.delay_to_OFDM)
+                channel.append(np.sum(array_response_RX[:, None, None, :] * array_response_TX[None, :, None, :] * path_const.T[None, None, :, :], axis=3)@path_gen.delay_to_OFDM)
         
         else: # TD channel
-            channel.append(array_response_RX[:, None, :] * np.conj(array_response_TX)[None, :, :] * (np.sqrt(raydata[i][c.OUT_PATH_RX_POW]) * np.exp(1j*np.deg2rad(raydata[i][c.OUT_PATH_PHASE])))[None, None, :])
+            channel.append(array_response_RX[:, None, :] * array_response_TX[None, :, :] * (np.sqrt(raydata[i][c.OUT_PATH_RX_POW]) * np.exp(1j*np.deg2rad(raydata[i][c.OUT_PATH_PHASE])))[None, None, :])
 
     return channel
 
 
-def array_response(ant_ind, theta, phi, kd):
+def array_response(ant_ind, theta, phi, kd, rotation=None):
+    theta = np.deg2rad(theta)
+    phi = np.deg2rad(phi)
+    
+    if rotation is not None:
+        theta, phi = rotate_angles(rotation, theta, phi)
+        
+        
     gamma = array_response_phase(theta, phi, kd)
     return np.exp(ant_ind@gamma.T)
     
 def array_response_phase(theta, phi, kd):
-    gamma_x = -1j*kd*np.sin(np.deg2rad(theta))*np.cos(np.deg2rad(phi))
-    gamma_y = -1j*kd*np.sin(np.deg2rad(theta))*np.sin(np.deg2rad(phi))
-    gamma_z = -1j*kd*np.cos(np.deg2rad(theta))
+    gamma_x = -1j*kd*np.sin(theta)*np.cos(phi)
+    gamma_y = -1j*kd*np.sin(theta)*np.sin(phi)
+    gamma_z = -1j*kd*np.cos(theta)
     return np.vstack([gamma_x, gamma_y, gamma_z]).T
  
 def ant_indices(panel_size):
@@ -130,8 +148,34 @@ def ant_indices(panel_size):
     gamma_z = np.repeat(np.arange(panel_size[2]), panel_size[0]*panel_size[1])
     return np.vstack([gamma_x, gamma_y, gamma_z]).T
 
+def rotate_angles(rotation, theta, phi): # Input all radians
+
+    rotation = np.deg2rad(rotation)
+
+    sin_alpha = np.sin(phi - rotation[2])
+    sin_beta = np.sin(rotation[1])
+    sin_gamma = np.sin(rotation[0])
+    cos_alpha = np.cos(phi - rotation[2])
+    cos_beta = np.cos(rotation[1])
+    cos_gamma = np.cos(rotation[0])
+    
+    sin_theta = np.sin(theta)
+    cos_theta = np.cos(theta)
+    
+    theta_new = np.arccos(cos_beta*cos_gamma*cos_theta 
+                          + sin_theta*(sin_beta*cos_gamma*cos_alpha-sin_gamma*sin_alpha)
+                          )
+    phi_new = np.angle(cos_beta*sin_theta*cos_alpha-sin_beta*cos_theta 
+                       + 1j*(cos_beta*sin_gamma*cos_theta 
+                             + sin_theta*(sin_beta*sin_gamma*cos_alpha + cos_gamma*sin_alpha))
+                       )
+    
+    return theta_new, phi_new
+
 class OFDM_PathGenerator:
     def __init__(self, OFDM_params, subcarriers):
+        self.OFDM_params = OFDM_params
+        
         if OFDM_params[c.PARAMSET_OFDM_PULSE] == 0: # No Pulse Shaping
             self.generate = getattr(self, 'no_pulse_shaping')
         else: # Pulse Shaping
@@ -158,12 +202,11 @@ class OFDM_PathGenerator:
         self.conv_interval = max(10., self.cyclic_prefix_length)
         self.upsampling_factor = float(OFDM_params[c.PARAMSET_OFDM_UPSAMPLING]) # Fixed
         self.t = np.arange(-self.conv_interval, self.cyclic_prefix_length + 6, 1/self.upsampling_factor).reshape(1, -1) # Constant
+        self.t_0 = int(self.conv_interval*self.upsampling_factor)
         
         # Low Pass Filter Selection
         if OFDM_params[c.PARAMSET_OFDM_LPF]:
             self.LPF = np.sinc(self.t).reshape(-1) # Fixed
-        else:
-            self.LPF = self.delta_func(self.t).reshape(-1) # Fixed
         
         self.downsample = (self.upsampling_factor*np.arange(self.conv_interval*2, self.conv_interval*2+self.cyclic_prefix_length)).astype(int)
         self.delay_to_OFDM = np.exp(-1j*2*np.pi/self.total_subcarriers*np.outer(self.delay_d, self.subcarriers))
@@ -180,9 +223,12 @@ class OFDM_PathGenerator:
         
         # Pulse - LPF convolution and channel generation
         pulse = self.pulse_fn(self.t-delay)
-        res = np.zeros((pulse.shape[0], len(self.LPF)*2-1), dtype=np.complex64)
-        for i in range(len(power)):
-            res[i, :] = convolve(pulse[i, :], self.LPF)
+        res = np.zeros((pulse.shape[0], self.t.shape[1]*2-1), dtype=np.complex64)
+        if self.OFDM_params[c.PARAMSET_OFDM_LPF]: # Sinc Pulse
+            for i in range(len(power)):
+                res[i, :] = convolve(pulse[i, :], self.LPF)
+        else: # Delta Pulse
+            res[:, self.t_0:-(self.t.shape[1]-self.t_0-1)] = pulse
         res /= np.max(np.abs(res), 1, keepdims=True) # Power normalization
         res = res[:, self.downsample]
         res *= np.sqrt(power/self.total_subcarriers) * np.exp(1j*np.deg2rad(phase)) # Power scaling
@@ -196,7 +242,7 @@ class OFDM_PathGenerator:
         rcos_filter[t_ind] = (np.pi/4)*np.sinc(1/(2*self.rolloff_factor))
         return rcos_filter
     
-    def delta_func(self, t):
-        pulse = np.zeros(t)
-        pulse[t==0] = 1
+    def delta_func(self, t, delta_loc):
+        pulse = np.zeros(np.prod(t.shape))
+        pulse[delta_loc] = 1
         return pulse
\ No newline at end of file
