@@ -44,8 +44,10 @@ PARAMSET_ANT_BS = 'bs_antenna'
 PARAMSET_ANT_UE = 'ue_antenna'
 PARAMSET_ANT_SHAPE = 'shape'
 PARAMSET_ANT_SPACING = 'spacing'
+PARAMSET_ANT_ROTATION = 'rotation'
 
 # INNER VARIABLES
+PARAMSET_ACTIVE_UE = 'active_UE'
 PARAMSET_SCENARIO_FIL = 'scenario_files'
 #PARAMSET_ACTIVE_USERS = 'active_user_idx'
 PARAMSET_ANT_BS_DIFF = 'different_bs_ants'
