@@ -73,13 +73,15 @@ def default_params():
               # BS Antenna Parameters
               c.PARAMSET_ANT_BS: {
                   c.PARAMSET_ANT_SHAPE: np.array([1, 4, 4]), # Antenna dimensions in X - Y - Z
-                  c.PARAMSET_ANT_SPACING: 0.5
+                  c.PARAMSET_ANT_SPACING: 0.5,
+                  #c.PARAMSET_ANT_ROTATION: np.array([0, 0, 0]) # Rotation around X - Y - Z axes
                   },
               
               # UE Antenna Parameters
               c.PARAMSET_ANT_UE: {
                   c.PARAMSET_ANT_SHAPE: np.array([1, 2, 2]), # Antenna dimensions in X - Y - Z
-                  c.PARAMSET_ANT_SPACING: 0.5
+                  c.PARAMSET_ANT_SPACING: 0.5,
+                  #c.PARAMSET_ANT_ROTATION: np.array([0, 0, 0]) # Rotation around X - Y - Z axes
                   }
               }
     return params
\ No newline at end of file
