@@ -1,8 +1,11 @@
 # -*- coding: utf-8 -*-
 """
-Created on Fri Nov 19 16:24:25 2021
+DeepMIMOv2 Python Implementation
 
-@author: Umt
+Description: Utilities
+
+Authors: Umut Demirhan, Ahmed Alkhateeb
+Date: 12/10/2021
 """
 
 import time
